#include "TestBed.h"

LRESULT CALLBACK MainWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam,
				 LPARAM lParam);
BOOL	GetEditLines( void);

HINSTANCE	hInstance;
HWND		hwndMain = NULL,
		hwndEdit = NULL;
int		cEditLines;
TCHAR		*szAppName,
		szAppPath[ MAX_PATH];

#pragma argsused
int WINAPI WinMain( HINSTANCE hInstanceIn, HINSTANCE hPrevInstance,
		LPSTR lpCmdLine, int nCmdShow)
{
WNDCLASS	wc;
MSG		msg;

::hInstance = hInstanceIn;

GetModuleFileName( hInstance, szAppPath, MAX_PATH);
szAppName = _tcsrchr( szAppPath, '\\') + 1;



wc.style		= CS_HREDRAW | CS_VREDRAW;
wc.lpfnWndProc		= MainWindowProc;
wc.cbClsExtra		= 0;
wc.cbWndExtra		= 0;
wc.hInstance		= hInstance;
wc.hIcon		= NULL;
wc.hCursor		= LoadCursor( NULL, IDC_ARROW);
wc.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
wc.lpszMenuName		= NULL;
wc.lpszClassName	= WNDCLASSNAME;

if (!RegisterClass( &wc))
	MbErr( TRUE, TEXT("Couldn't register class"));

hwndMain = CreateWindow( WNDCLASSNAME, szAppName, WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInstance, 0);

if (!hwndMain)
	MbErr( TRUE, TEXT("Couln't create window"));

ShowWindow( hwndMain, SW_SHOWNORMAL);
UpdateWindow( hwndMain);

while (GetMessage( &msg, NULL, 0, 0))
	{
	TranslateMessage( &msg);
	DispatchMessage( &msg);
	}
return msg.wParam;
}


LRESULT CALLBACK MainWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam,
				 LPARAM lParam)
{
RECT		rect;
HDC		hdc;
LOGFONT		lf;
static HFONT	hFont = NULL;

switch (uMsg)
	{
	case WM_CREATE:
		GetClientRect( hwnd, &rect);
		hwndEdit = CreateWindow( TEXT("EDIT"), TEXT(""),
			ES_MULTILINE|ES_NOHIDESEL|ES_READONLY|
			WS_VSCROLL|WS_HSCROLL|WS_VISIBLE|WS_CHILD,
			0, 0, rect.right-rect.left, rect.bottom-rect.top,
			hwnd, NULL, hInstance, 0);
		if (!hwndEdit)
			{
			MbErr( FALSE, TEXT("Couldn't create edit control"));
			return -1;
			}

		lf.lfHeight		= 0;
		lf.lfWidth		= 0;
		lf.lfEscapement		= 0;
		lf.lfOrientation	= 0;
		lf.lfWeight		= FW_DONTCARE;
		lf.lfItalic		= FALSE;
		lf.lfUnderline		= FALSE;
		lf.lfStrikeOut		= FALSE;
		lf.lfCharSet		= DEFAULT_CHARSET;
		lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
		lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
		lf.lfQuality		= DEFAULT_QUALITY;
		lf.lfPitchAndFamily	= FIXED_PITCH;
		lf.lfFaceName[0]	= '\0';

		hFont = CreateFontIndirect( &lf);

		if (!hFont)
			{
			MbErr( FALSE, TEXT("Couldn't create font"));
			return -1;
			}

		SendMessage( hwndEdit, WM_SETFONT, (WPARAM)hFont, TRUE);

		if (!GetEditLines())
			return -1;

		if (SetTimer( hwnd, 1, 50, NULL) == 0)
			{
			MbErr( FALSE, TEXT("Couldn't create timer"));
			return -1;
			}
		return 0;

	case WM_SIZE:
		GetClientRect( hwnd, &rect);
		MoveWindow( hwndEdit, 0, 0, rect.right - rect.left,
			rect.bottom - rect.top, TRUE);
		GetEditLines();
		break;

	case WM_DESTROY:
		if (hFont) DeleteObject( hFont);
		DestroyWindow( hwndEdit);
		PostQuitMessage( 0);
		return 0;

	case WM_TIMER:
		if (wParam == 1)
			{
			KillTimer( hwnd, 1);
			TestFunction();
			return 0;
			}
		break;
	}

return DefWindowProc( hwnd, uMsg, wParam, lParam);
}



void MbErr( BOOL bKillApp, TCHAR *szMsg)
{
MessageBox( NULL, szMsg, TEXT("Error"), MB_OK|MB_ICONERROR);
if (bKillApp) ExitProcess( 1);
}


int logargs( TCHAR *szFormat, va_list args)
{
int		len, curLine, lineCount;
TCHAR		szBuf[ BUFSIZE], szBuf2[ BUFSIZE], *pSrc, *pDest;

if ((!szFormat) || (!_tcslen(szFormat))) return 0;

len = (int)SendMessage( hwndEdit, WM_GETTEXTLENGTH, 0, 0);

SendMessage( hwndEdit, EM_SETSEL, len, len+1);

pSrc = szFormat;
pDest = szBuf2;
while (*pSrc && (pDest - szBuf2 < BUFSIZE-2))
	{
	if (*pSrc != '\n')
		*pDest = *pSrc;
	else	{
		*pDest = '\x0D';
		pDest++;
		*pDest = '\x0A';
		}
	pSrc++;
	pDest++;
	}
*pDest = '\0';
if (pDest-szBuf2 >= BUFSIZE-2)
	return 0;


len = _vsntprintf( szBuf, BUFSIZE-2, szBuf2, args);
if (len<BUFSIZE-2)
	SendMessage( hwndEdit, EM_REPLACESEL, FALSE, (LPARAM)szBuf);
else	return 0;


curLine = SendMessage( hwndEdit, EM_GETFIRSTVISIBLELINE, 0, 0);
lineCount = SendMessage( hwndEdit, EM_GETLINECOUNT, 0, 0);

if (lineCount - curLine > cEditLines)
	SendMessage( hwndEdit, EM_LINESCROLL, 0,
		lineCount - cEditLines - curLine);

return len;
}


int	log( TCHAR *szFormat, ...)
{
va_list	args;
int	len=0;

if (szFormat && _tcslen(szFormat))
	{
	va_start( args, szFormat);
	len = logargs( szFormat, args);
	va_end( args);
	}
return len;
}


int	logln( TCHAR *szFormat, ...)
{
va_list	args;
int	len=0;

if (szFormat && _tcslen(szFormat))
	{
	va_start( args, szFormat);
	len = logargs( szFormat, args);
	va_end( args);
	}
return (log(TEXT("\n"))?len+2:len);
}

int	logwe( TCHAR *szFormat, ...)
{
va_list	args;
int	len=0;
DWORD	lastError = GetLastError();

if (szFormat && _tcslen(szFormat))
	{
	va_start( args, szFormat);
	len = logerrargs( lastError, szFormat, args);
	va_end( args);
	}
return len;
}



int	logerr( DWORD dwWinError, TCHAR *szFormat, ...)
{
va_list	args;
int	len;

if (szFormat && _tcslen(szFormat))
	{
	va_start( args, szFormat);
	len = logerrargs( dwWinError, szFormat, args);
	va_end( args);
	}

return len;
}



int	logerrargs( DWORD dwWinError, TCHAR *szFormat, va_list args)
{
TCHAR	szBuf[ BUFSIZE];
int	len, len2;

_tcscpy( szBuf, TEXT("\n\n*** Error ***\n"));
len = _tcslen( szBuf);

if (szFormat && _tcslen(szFormat))
	{
	if ((len2=_tcslen( szFormat)) + len >= BUFSIZE - 2)
		return 0;
	_tcscat( szBuf, szFormat);
	len += len2;
	if (szBuf[len]!='\n')
		{
		if (len+1 >= BUFSIZE -2)
			return 0;
		_tcscat( szBuf, TEXT("\n"));
		len++;
		}
	}

len2 =_sntprintf( &szBuf[len], BUFSIZE-2-len, TEXT("Windows says: (%u) "),
	dwWinError);

if (!len2) return 0;
len = _tcslen( szBuf);


len2 = FormatMessage(
	FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
	0, dwWinError, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
	&szBuf[len], BUFSIZE - len, NULL);

if (len2 == BUFSIZE - len)
	return 0;

return logargs( szBuf, args);
}



BOOL	GetEditLines( void)
{
HDC		hdc;
TEXTMETRIC	tm;
RECT		rect;

GetClientRect( hwndEdit, &rect);

if ((hdc = GetDC( hwndEdit)) == NULL)
	MbErr( TRUE, TEXT("Couldn't get DC for edit control"));

if (!GetTextMetrics( hdc, &tm))
	MbErr( TRUE, TEXT("Couldn't GetTextMetrics for edit control"));

cEditLines = ((rect.bottom - rect.top) / tm.tmHeight) +
	     ( ((rect.bottom - rect.top) % tm.tmHeight) ? 1:0);

ReleaseDC( hwndEdit, hdc);
return TRUE;
}


HWND	GetMainWnd( void)
{ return hwndMain; }


